d.order <- function(n, r, distfun, densityfun) {
  f <- function(x) NULL
  con <- exp(lgamma(n + 1) - lgamma(r) - lgamma(n - r + 1))
  c1 <- call(deparse(substitute(distfun)), as.name("x"))
  c2 <- call("^", call("-", 1, c1), n - r)
  c3 <- call("^", c1, r - 1)
  c4 <- call("*", con, call("*", c2, call("*", c3,
             call(deparse(substitute(densityfun)), as.name("x")))))
  body(f) <- c4
  f
}

make.integrand <- function (n, r, cdf, pdf) {  ## R only.
  cdf <- substitute(cdf)
  if (is.name(cdf)) cdf <- call(deparse(cdf), as.name("x"))
  pdf <- substitute(pdf)
  if (is.name(pdf)) pdf <- call(deparse(pdf), as.name("x"))
  con <- round(exp(lgamma(n+1) - lgamma(r) - lgamma(n-r+1)))
  function(x) {
    P <- eval(cdf); D <- eval(pdf)
    x * con * P^(r - 1) * (1 - P)^(n - r) * D
  }
}

E.order <- function(n, cdf, pdf, lower = -10, upper = 10, ...)
{
  Ex <- numeric(n)
  for(i in 1:n) {
    f <- eval(call("make.integrand", n, i,
                   substitute(cdf), substitute(pdf)))
    Ex[i] <- integrate(f, lower, upper, ...)[[1]]
  }
  Ex
}


# Another approach

make.integrand <- function(n, r, cdf, pdf, ...) {
  con <- round(exp(lgamma(n+1) - lgamma(r) - lgamma(n-r+1)))
  f <- eval(substitute(function(x, cdf, pdf) {
                  P <- cdf(x)
                  K * x * P^r1 * (1 - P)^nr * pdf(x)
                  }, list(K = con, r1 = r - 1, nr = n - r)))
  dots <- list(...)
  if(length(dots)) {
    formals(cdf)[names(dots)] <- unlist(dots)
    formals(pdf)[names(dots)] <- unlist(dots)
  }
  formals(f)$cdf <- cdf; formals(f)$pdf <- pdf
  environment(f) <- .GlobalEnv
  f
}
E.order <- function(n, cdf, pdf, lower = -10, upper = 10, ...)
{
  Ex <- numeric(n)
  for(i in 1:n) {
    f <- make.integrand(n, i, cdf, pdf, ...)
    Ex[i] <- integrate(f, lower = lower, upper = upper)[[1]]
  }
  Ex
}

make.integrand <- function(n, r, cdf, pdf) {
  con <- round(exp(lgamma(n+1) - lgamma(r) - lgamma(n-r+1)))
  function(x, ...) {
    P <- cdf(x, ...)
    x * con * P^(r-1) * (1-P)^(n-r) * pdf(x, ...)
  }
}

E.order <- function(n, cdf, pdf, lower = -10, upper = 10, ...)
{
  Ex <- numeric(n)
  for(i in 1:n) {
    f <- make.integrand(n, i, cdf, pdf)
    Ex[i] <- integrate(f, lower = lower, upper = upper,
                       ...)[[1]]
  }
  Ex
}
